@@ -4,21 +4,21 @@ from __future__ import annotations
 from dataclasses import dataclass, field
 
 from homeassistant.components.sensor import (
-    STATE_CLASS_MEASUREMENT,
-    STATE_CLASS_TOTAL_INCREASING,
+    SensorDeviceClass,
     SensorEntity,
     SensorEntityDescription,
+    SensorStateClass,
 )
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
-    DEVICE_CLASS_ENERGY,
-    DEVICE_CLASS_POWER,
-    DEVICE_CLASS_VOLTAGE,
     ELECTRIC_POTENTIAL_VOLT,
     ENERGY_KILO_WATT_HOUR,
     ENERGY_WATT_HOUR,
     POWER_WATT,
 )
+from homeassistant.core import HomeAssistant
 from homeassistant.helpers.entity import DeviceInfo
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from .const import DOMAIN
 
@@ -55,8 +55,8 @@ TREND_SENSORS: tuple[SmappeePollingSensorEntityDescription, ...] = (
         name="Total consumption - Active power",
         native_unit_of_measurement=POWER_WATT,
         sensor_id="total_power",
-        device_class=DEVICE_CLASS_POWER,
-        state_class=STATE_CLASS_MEASUREMENT,
+        device_class=SensorDeviceClass.POWER,
+        state_class=SensorStateClass.MEASUREMENT,
         local_polling=True,  # both cloud and local
     ),
     SmappeePollingSensorEntityDescription(
@@ -64,40 +64,40 @@ TREND_SENSORS: tuple[SmappeePollingSensorEntityDescription, ...] = (
         name="Always on - Active power",
         native_unit_of_measurement=POWER_WATT,
         sensor_id="alwayson",
-        device_class=DEVICE_CLASS_POWER,
-        state_class=STATE_CLASS_MEASUREMENT,
+        device_class=SensorDeviceClass.POWER,
+        state_class=SensorStateClass.MEASUREMENT,
     ),
     SmappeePollingSensorEntityDescription(
         key="power_today",
         name="Total consumption - Today",
         native_unit_of_measurement=ENERGY_WATT_HOUR,
         sensor_id="power_today",
-        device_class=DEVICE_CLASS_ENERGY,
-        state_class=STATE_CLASS_TOTAL_INCREASING,
+        device_class=SensorDeviceClass.ENERGY,
+        state_class=SensorStateClass.TOTAL_INCREASING,
     ),
     SmappeePollingSensorEntityDescription(
         key="power_current_hour",
         name="Total consumption - Current hour",
         native_unit_of_measurement=ENERGY_WATT_HOUR,
         sensor_id="power_current_hour",
-        device_class=DEVICE_CLASS_ENERGY,
-        state_class=STATE_CLASS_TOTAL_INCREASING,
+        device_class=SensorDeviceClass.ENERGY,
+        state_class=SensorStateClass.TOTAL_INCREASING,
     ),
     SmappeePollingSensorEntityDescription(
         key="power_last_5_minutes",
         name="Total consumption - Last 5 minutes",
         native_unit_of_measurement=ENERGY_WATT_HOUR,
         sensor_id="power_last_5_minutes",
-        device_class=DEVICE_CLASS_ENERGY,
-        state_class=STATE_CLASS_TOTAL_INCREASING,
+        device_class=SensorDeviceClass.ENERGY,
+        state_class=SensorStateClass.TOTAL_INCREASING,
     ),
     SmappeePollingSensorEntityDescription(
         key="alwayson_today",
         name="Always on - Today",
         native_unit_of_measurement=ENERGY_WATT_HOUR,
         sensor_id="alwayson_today",
-        device_class=DEVICE_CLASS_ENERGY,
-        state_class=STATE_CLASS_TOTAL_INCREASING,
+        device_class=SensorDeviceClass.ENERGY,
+        state_class=SensorStateClass.TOTAL_INCREASING,
     ),
 )
 REACTIVE_SENSORS: tuple[SmappeeSensorEntityDescription, ...] = (
@@ -106,8 +106,8 @@ REACTIVE_SENSORS: tuple[SmappeeSensorEntityDescription, ...] = (
         name="Total consumption - Reactive power",
         native_unit_of_measurement=POWER_WATT,
         sensor_id="total_reactive_power",
-        device_class=DEVICE_CLASS_POWER,
-        state_class=STATE_CLASS_MEASUREMENT,
+        device_class=SensorDeviceClass.POWER,
+        state_class=SensorStateClass.MEASUREMENT,
     ),
 )
 SOLAR_SENSORS: tuple[SmappeePollingSensorEntityDescription, ...] = (
@@ -116,8 +116,8 @@ SOLAR_SENSORS: tuple[SmappeePollingSensorEntityDescription, ...] = (
         name="Total production - Active power",
         native_unit_of_measurement=POWER_WATT,
         sensor_id="solar_power",
-        device_class=DEVICE_CLASS_POWER,
-        state_class=STATE_CLASS_MEASUREMENT,
+        device_class=SensorDeviceClass.POWER,
+        state_class=SensorStateClass.MEASUREMENT,
         local_polling=True,  # both cloud and local
     ),
     SmappeePollingSensorEntityDescription(
@@ -125,16 +125,16 @@ SOLAR_SENSORS: tuple[SmappeePollingSensorEntityDescription, ...] = (
         name="Total production - Today",
         native_unit_of_measurement=ENERGY_WATT_HOUR,
         sensor_id="solar_today",
-        device_class=DEVICE_CLASS_ENERGY,
-        state_class=STATE_CLASS_TOTAL_INCREASING,
+        device_class=SensorDeviceClass.ENERGY,
+        state_class=SensorStateClass.TOTAL_INCREASING,
     ),
     SmappeePollingSensorEntityDescription(
         key="solar_current_hour",
         name="Total production - Current hour",
         native_unit_of_measurement=ENERGY_WATT_HOUR,
         sensor_id="solar_current_hour",
-        device_class=DEVICE_CLASS_ENERGY,
-        state_class=STATE_CLASS_TOTAL_INCREASING,
+        device_class=SensorDeviceClass.ENERGY,
+        state_class=SensorStateClass.TOTAL_INCREASING,
     ),
 )
 VOLTAGE_SENSORS: tuple[SmappeeVoltageSensorEntityDescription, ...] = (
@@ -143,8 +143,8 @@ VOLTAGE_SENSORS: tuple[SmappeeVoltageSensorEntityDescription, ...] = (
         name="Phase voltages - A",
         native_unit_of_measurement=ELECTRIC_POTENTIAL_VOLT,
         sensor_id="phase_voltage_a",
-        device_class=DEVICE_CLASS_VOLTAGE,
-        state_class=STATE_CLASS_MEASUREMENT,
+        device_class=SensorDeviceClass.VOLTAGE,
+        state_class=SensorStateClass.MEASUREMENT,
         phase_types={"ONE", "TWO", "THREE_STAR", "THREE_DELTA"},
     ),
     SmappeeVoltageSensorEntityDescription(
@@ -152,8 +152,8 @@ VOLTAGE_SENSORS: tuple[SmappeeVoltageSensorEntityDescription, ...] = (
         name="Phase voltages - B",
         native_unit_of_measurement=ELECTRIC_POTENTIAL_VOLT,
         sensor_id="phase_voltage_b",
-        device_class=DEVICE_CLASS_VOLTAGE,
-        state_class=STATE_CLASS_MEASUREMENT,
+        device_class=SensorDeviceClass.VOLTAGE,
+        state_class=SensorStateClass.MEASUREMENT,
         phase_types={"TWO", "THREE_STAR", "THREE_DELTA"},
     ),
     SmappeeVoltageSensorEntityDescription(
@@ -161,8 +161,8 @@ VOLTAGE_SENSORS: tuple[SmappeeVoltageSensorEntityDescription, ...] = (
         name="Phase voltages - C",
         native_unit_of_measurement=ELECTRIC_POTENTIAL_VOLT,
         sensor_id="phase_voltage_c",
-        device_class=DEVICE_CLASS_VOLTAGE,
-        state_class=STATE_CLASS_MEASUREMENT,
+        device_class=SensorDeviceClass.VOLTAGE,
+        state_class=SensorStateClass.MEASUREMENT,
         phase_types={"THREE_STAR"},
     ),
     SmappeeVoltageSensorEntityDescription(
@@ -170,8 +170,8 @@ VOLTAGE_SENSORS: tuple[SmappeeVoltageSensorEntityDescription, ...] = (
         name="Line voltages - A",
         native_unit_of_measurement=ELECTRIC_POTENTIAL_VOLT,
         sensor_id="line_voltage_a",
-        device_class=DEVICE_CLASS_VOLTAGE,
-        state_class=STATE_CLASS_MEASUREMENT,
+        device_class=SensorDeviceClass.VOLTAGE,
+        state_class=SensorStateClass.MEASUREMENT,
         phase_types={"ONE", "TWO", "THREE_STAR", "THREE_DELTA"},
     ),
     SmappeeVoltageSensorEntityDescription(
@@ -179,8 +179,8 @@ VOLTAGE_SENSORS: tuple[SmappeeVoltageSensorEntityDescription, ...] = (
         name="Line voltages - B",
         native_unit_of_measurement=ELECTRIC_POTENTIAL_VOLT,
         sensor_id="line_voltage_b",
-        device_class=DEVICE_CLASS_VOLTAGE,
-        state_class=STATE_CLASS_MEASUREMENT,
+        device_class=SensorDeviceClass.VOLTAGE,
+        state_class=SensorStateClass.MEASUREMENT,
         phase_types={"TWO", "THREE_STAR", "THREE_DELTA"},
     ),
     SmappeeVoltageSensorEntityDescription(
@@ -188,14 +188,18 @@ VOLTAGE_SENSORS: tuple[SmappeeVoltageSensorEntityDescription, ...] = (
         name="Line voltages - C",
         native_unit_of_measurement=ELECTRIC_POTENTIAL_VOLT,
         sensor_id="line_voltage_c",
-        device_class=DEVICE_CLASS_VOLTAGE,
-        state_class=STATE_CLASS_MEASUREMENT,
+        device_class=SensorDeviceClass.VOLTAGE,
+        state_class=SensorStateClass.MEASUREMENT,
         phase_types={"THREE_STAR", "THREE_DELTA"},
     ),
 )
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up the Smappee sensor."""
     smappee_base = hass.data[DOMAIN][config_entry.entry_id]
 
@@ -250,9 +254,10 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
                     description=SmappeeSensorEntityDescription(
                         key="load",
                         name=measurement.name,
+                        native_unit_of_measurement=POWER_WATT,
                         sensor_id=measurement_id,
-                        device_class=DEVICE_CLASS_POWER,
-                        state_class=STATE_CLASS_MEASUREMENT,
+                        device_class=SensorDeviceClass.POWER,
+                        state_class=SensorStateClass.MEASUREMENT,
                     ),
                 )
                 for measurement_id, measurement in service_location.measurements.items()
@@ -295,7 +300,7 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
                         ),
                         native_unit_of_measurement=channel.get("uom"),
                         sensor_id=f"{sensor_id}-{channel.get('channel')}",
-                        state_class=STATE_CLASS_MEASUREMENT,
+                        state_class=SensorStateClass.MEASUREMENT,
                     ),
                 )
                 for sensor_id, sensor in service_location.sensors.items()
@@ -314,12 +319,12 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
                         name=f"{actuator.name} - energy today",
                         native_unit_of_measurement=ENERGY_KILO_WATT_HOUR,
                         sensor_id=actuator_id,
-                        device_class=DEVICE_CLASS_ENERGY,
-                        state_class=STATE_CLASS_TOTAL_INCREASING,
+                        device_class=SensorDeviceClass.ENERGY,
+                        state_class=SensorStateClass.TOTAL_INCREASING,
                     ),
                 )
                 for actuator_id, actuator in service_location.actuators.items()
-                if actuator.type == "SWITCH"
+                if actuator.type == "SWITCH" and not service_location.local_polling
             ]
         )
 
@@ -383,7 +388,7 @@ class SmappeeSensor(SensorEntity):
             sw_version=self._service_location.firmware_version,
         )
 
-    async def async_update(self):
+    async def async_update(self) -> None:
         """Get the latest data from Smappee and update the state."""
         await self._smappee_base.async_update()
 
